# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.


import asyncio
from pyromod.utils import patch, patchable
from pyrogram.types import Message


@patch(Message)
class Message:
    @patchable
    async def react(self, text: str, del_in: int = 0, **kwargs):
        reply = await self.reply(text, quote=True, **kwargs)
        if del_in:
            await asyncio.sleep(del_in)
            await reply.delete()



# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.


from ..client import Stark
from ..config import check_environment, OWNER_ID
from pyrogram.errors import PeerIdInvalid
from pyrogram.types import CallbackQuery, InlineKeyboardMarkup, InlineKeyboardButton

module = check_environment()


@Stark.cmd('start')
async def start_func(bot: Stark, msg):
    try:
        text = await replace(module.START, msg, bot)
        if send_buttons():
            await msg.react(text, reply_markup=InlineKeyboardMarkup(main_buttons))
        else:
            await msg.react(text)
    except AttributeError:
        pass


@Stark.cmd('help')
async def help_func(bot, msg):
    try:
        text = await replace(module.HELP, msg, bot)
        if send_buttons():
            await msg.react(text, reply_markup=InlineKeyboardMarkup(home_button))
        else:
            await msg.react(text)
    except AttributeError:
        pass


@Stark.cmd('about')
async def about_func(bot, msg):
    try:
        text = await replace(module.ABOUT, msg, bot)
        if send_buttons():
            await msg.react(text, reply_markup=InlineKeyboardMarkup(home_button))
        else:
            await msg.react(text)
    except AttributeError:
        pass


@Stark.cmd('id')
async def id_func(_, msg):
    if msg.chat.type == 'private':
        await msg.react("Your ID is `{}`".format(msg.from_user.id))
    else:
        if msg.reply_to_message:
            await msg.react("{}'s ID is `{}`".format(msg.reply_to_message.from_user.first_name, msg.reply_to_message.from_user.id))
        else:
            await msg.react("Chat ID is `{}` \n\nYour ID is `{}`".format(msg.chat.id, msg.from_user.id))


@Stark.callback(in_list=['home', 'about', 'help'])
async def basic_cb(bot: Stark, cb: CallbackQuery):
    chat_id = cb.from_user.id
    message_id = cb.message.message_id
    if cb.data == 'home':
        text = await replace(module.START, cb.message, bot)
        buttons = main_buttons
    elif cb.data == 'about':
        text = await replace(module.ABOUT, cb.message, bot)
        buttons = home_button
    else:
        text = await replace(module.HELP, cb.message, bot)
        buttons = home_button
    await bot.edit_message_text(
        chat_id=chat_id,
        message_id=message_id,
        text=text,
        reply_markup=InlineKeyboardMarkup(buttons),
        disable_web_page_preview=True,
    )


home_button = [
    [InlineKeyboardButton(text="🏠 Return Home 🏠", callback_data="home")]
]

main_buttons = [
    [InlineKeyboardButton("✨ Bot Status and More Bots ✨", url="https://t.me/StarkBots/7")],
    [
        InlineKeyboardButton("How to Use ❔", callback_data="help"),
        InlineKeyboardButton("🎪 About 🎪", callback_data="about")
    ],
    [InlineKeyboardButton("♥ More Amazing bots ♥", url="https://t.me/StarkBots")],
]


async def replace(m, msg, bot):
    # I do know str.format() but that'll be unnecessary execution of multiple things and anyway these are optional.
    # For same reason, IF statements are used below. Replies should be fast no matter what.
    # Also, prevents
    if '{user}' in m:
        m = m.replace("{user}", msg.from_user.first_name)
    if '{bot}' in m:
        m = m.replace("{bot}", (await bot.get_me()).first_name)
    if '{user_mention}' in m:
        m = m.replace("{user_mention}", msg.from_user.mention)
    if '{bot_mention}' in m:
        m = m.replace("{bot_mention}", (await bot.get_me()).mention)
    if '{owner}' in m:
        try:
            owner = (await bot.get_users(int(OWNER_ID))).mention
        except PeerIdInvalid:
            owner = '@StarkBots'
        m = m.replace("{owner}", owner)
    return m


def send_buttons():
    if 'BUTTONS' in module.__dict__ and module.BUTTONS:
        return True
    return False


# To-DO
# Don't repeat replace() function everywhere
# Better management for 'module' variable



# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.


import os
from .logger import logger
from importlib import import_module
from dotenv import load_dotenv

load_dotenv('.env')

API_ID = os.environ.get("API_ID", 0)
API_HASH = os.environ.get("API_HASH", None)
BOT_TOKEN = os.environ.get("BOT_TOKEN", None)
CMD_PREFIXES = list(os.environ.get("CMD_PREFIX", "/").strip())
OWNER_ID = os.environ.get("OWNER_ID", 1946995626)
TIMEZONE = os.environ.get("TIMEZONE", "Asia/Kolkata")

DATABASE_URL = os.environ.get("DATABASE_URL", None)
if DATABASE_URL:
    DATABASE_URL = DATABASE_URL.replace("postgres", "postgresql")
REDIS_URL = os.environ.get("REDIS_URL", None)
REDIS_PASSWORD = os.environ.get("REDIS_PASSWORD", None)


def check_environment():
    if not API_ID:
        logger.critical("No API_ID found. Exiting...")
        raise SystemExit
    if not API_HASH:
        logger.critical("No API_HASH found. Exiting...")
        raise SystemExit
    if not BOT_TOKEN:
        logger.critical("No BOT_TOKEN found. Exiting...")
        raise SystemExit
    try:
        int(API_ID)
    except ValueError:
        logger.critical("API_ID is not a valid integer. Exiting...")
        raise SystemExit
    try:
        int(OWNER_ID)
    except ValueError:
        logger.warn("OWNER_ID is not a valid integer. Exiting...")
        raise SystemExit
    if os.path.exists('data.py'):
        module = import_module('data')
    else:
        logger.warn("No 'data.py' found. Default values will be used.")
        module = import_module('pystark.data')
    return module



# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.


from pyrogram import filters as f
from ..config import CMD_PREFIXES, OWNER_ID
from pyrogram.methods.decorators.on_message import OnMessage


class Command(OnMessage):
    @staticmethod
    def command(
        cmd: str = None,
        # help_string: str = "",
        group: int = 0,
        owner_only: bool = False,
        extra_filters=None
    ):
        filters_ = f.command(cmd, prefixes=CMD_PREFIXES)
        if owner_only:
            filters_ = filters_ & f.user(OWNER_ID)
        if extra_filters:
            filters_ = filters_ & extra_filters
        decorator = OnMessage.on_message(filters_, group)
        return decorator

    cmd = command



# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.


from pyrogram import filters as f
from pyrogram.methods.decorators.on_callback_query import OnCallbackQuery


class Callback(OnCallbackQuery):
    @staticmethod
    def callback(
        query_string: str = None,
        in_list: list = None,
        startswith: bool = False,
        group: int = 0,
        filters=None
    ):
        if in_list:
            cmd_filter = f.create(lambda _, __, query: query.data in in_list)
        else:
            if not startswith:
                cmd_filter = f.create(lambda _, __, query: query.data == query_string)
            else:
                cmd_filter = f.create(lambda _, __, query: query.data.startswith(query_string))
        if not filters:
            filters = cmd_filter
        else:
            filters = cmd_filter & filters
        decorator = OnCallbackQuery.on_callback_query(filters, group)
        return decorator



# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.


from setuptools import setup, find_packages

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

with open("requirements.txt", encoding="utf-8") as r:
    install_requires = [i.strip() for i in r]

packages = find_packages()

setup(
    name='PyStark',
    packages=packages,
    version='0.2.0',
    license='GPLv3+',
    description='An incomplete add-on extension to Pyrogram for personal use.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='StarkBotsIndustries',
    author_email='starkbotsindustries@gmail.com',
    url='https://github.com/StarkBotsIndustries/PyStark',
    download_url='https://github.com/StarkBotsIndustries/PyStark/releases/latest',
    keywords=['telegram', 'bot', 'pyrogram', 'python'],
    install_requires=install_requires,
    zip_safe=False,
    python_requires="~=3.6",
    dependency_links=['https://github.com/pyrogram/pyrogram/tarball/master'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Natural Language :: English',
        'Topic :: Communications :: Chat',
        'Topic :: Education :: Testing',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    project_urls={
        "Community": "https://t.me/StarkBots",
        "Support": "https://t.me/StarkBotsChat",
    },
    entry_points={
        'console_scripts': [
            'pystark = pystark.console:main',
        ],
    },
)



# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.


import sqlalchemy
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, scoped_session
from ..config import DATABASE_URL

if not DATABASE_URL:
    pass


def start() -> scoped_session:
    engine = create_engine(DATABASE_URL)
    Base.metadata.bind = engine
    Base.metadata.create_all(engine)
    return scoped_session(sessionmaker(bind=engine, autoflush=False))


Base = declarative_base()
# Type-Hinting is wrong here, but a temporary way to get Hints for Session object
Session: sqlalchemy.orm.Session = start()



# PyStark - Python add-on extension to Pyrogram
# Copyright (C) 2021-2022 Stark Bots <https://github.com/StarkBotsIndustries>
#
# This file is part of PyStark.
#
# PyStark is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyStark is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyStark. If not, see <https://www.gnu.org/licenses/>.


import os
import struct
import logging
from .logger import logger
from .decorators import Mechanism
from pyrogram import Client, idle
from importlib import import_module
from inspect import getmembers, isfunction
from .config import API_ID, API_HASH, BOT_TOKEN, check_environment
from pyrogram.errors import (
    ApiIdInvalid,
    AccessTokenInvalid,
    AuthKeyDuplicated,
    AuthKeyUnregistered,
    UserDeactivated,
)


class Stark(Client, Mechanism):
    support = "@StarkBotsChat"

    def __init__(self):
        check_environment()
        super().__init__(
            ":memory:",
            api_id=int(API_ID),
            api_hash=API_HASH,
            bot_token=BOT_TOKEN,
        )

    def start(self):
        try:
            super().start()
            return
        except ApiIdInvalid:
            logger.critical("API_ID and API_HASH combination is incorrect.")
        except AccessTokenInvalid:
            logger.critical("BOT_TOKEN is invalid.")
        except (AuthKeyUnregistered, AuthKeyDuplicated, struct.error):
            logger.critical("Your STRING_SESSION is invalid. Please terminate it and generate a new one.")
        except UserDeactivated:
            logger.critical(f"Account deleted. Time for me to rest.")
        except KeyboardInterrupt:
            logger.critical("Keyboard Interrupt. Exiting..")
        logger.info("For support visit {}".format(Stark.support))
        raise SystemExit

    @staticmethod
    def log(message, level=logging.INFO):
        logger.log(level, message)

    def activate(self, plugins: str = None, default_plugins: bool = True):
        if not plugins:
            plugins = 'plugins'
        self.start()
        self.load_modules(plugins)
        if default_plugins:
            from pystark import plugins
            self.load_modules(plugins.__path__[0])
        logger.info("{} is now running...".format('@' + self.get_me().username))
        idle()
        self.stop()
        logger.info("Bot has stopped working")

    def load_modules(self, plugins):
        modules = self.list_modules(plugins)
        if not modules:
            return
        for module in modules:
            if module.startswith("__"):
                return
            if 'pystark' in plugins:
                plugins = 'pystark.plugins'
            mod = import_module(plugins+'.'+module)
            funcs = [func for func, _ in getmembers(mod, isfunction)]
            for func in funcs:
                try:
                    for handler, group in getattr(mod, func).handlers:
                        self.add_handler(handler, group)
                except AttributeError:  # Other Functions shouldn't be included
                    pass
            if module == "basic":
                logger.info("Loaded - Default Plugins")
            else:
                logger.info("Loaded - {}.py".format(module))

    @staticmethod
    def list_modules(directory):
        try:
            return [file[:-3] for file in os.listdir(directory.replace('.', '/')) if file.endswith(".py")]
        except FileNotFoundError:
            logger.warn("No Custom Plugins Found!")
            return
